<?php

/**
 * @file
 * Contains \Drupal\chessboard_filter\ChessboardBulletinBoardCodeTagContentFilter.
 */

namespace Drupal\chessboard_filter;

use Drupal\chessboard\ChessboardFilterInterface as FilterInterface;
use Drupal\chessboard_flexible\PiecePlacement;

/**
 * Provides chessboard BBCode tag rendering.
 */
class ChessboardBulletinBoardCodeTagContentFilter implements FilterInterface{

  /**
  * Filter options.
   *
   * @var array
   */
  protected $options;

  /**
   * Constructs a Drupal\chessboard_filter\ChessboardBulletinBoardCodeTagContentFilter object.
   *
   * @param array $options
   *   A filter options array.
   */
  public function __construct(array $options) {
    $this->options = $options;
  }

  /**
   * Returns the result of filtering $value.
   *
   * @param string $value
   *   The piece placement field.
   * @return array
   *   A render <code>array</code> containing the replacement text.
   */
  public function filter($value) {
    $item = array(
      'value' => $value,
    );
    $item = $this->chessboardFilterProcessChessboardNotation($item);
    $element = array(
      '#attached' => array(
        'library' => array('chessboard_flexible/drupal.chessboard_flexible'),
      ),
      '#theme_wrappers' => array('container'),
      '#attributes' => array('class' => array('chessboard')),
      'chessboard' => array(
        '#type' => 'chessboard_flexible_diagram',
        '#piece_placement' => $item['piece_placement'],
        '#border' => $item['border'],
        '#square_color_first' => $item['square_color_first'],
      ),
    );
    return $element;
  }

  private function chessboardFilterProcessChessboardNotation($item) {
    preg_match('@^\s*(?:\((.*?)\))?(.*)$@s', $item['value'], $matches);

    // Parameter Parsing
    //--------------------------------
    $params =& $matches[1];

    // Number of files: any integer.
    if (preg_match('@[0-9]+@', $params, $m)) {
      $item['file_max'] = $m[0];
    }

    // Color of the upper left square: [l]ight or [d]ark.
    if (strpos($params, 'd') !== FALSE) {
      $item['square_color_first'] = 1;
    }
    elseif (strpos($params, 'l') !== FALSE) {
      $item['square_color_first'] = 0;
    }

    // Borders.
    $item['border'] = array('T' => (strpos($params, 'T') !== FALSE),
                    'B' => (strpos($params, 'B') !== FALSE),
                    'L' => (strpos($params, 'L') !== FALSE),
                    'R' => (strpos($params, 'R') !== FALSE));

    $item += $this->options;

    // Render the board in XHTML syntax
    //----------------------------------
    $board_value =& $matches[2];
    $board_value = strip_tags($board_value);
    // Ignore unknown characters.
    $board_value = preg_replace('@[^-0-9KQBNRPACkqbnrpacx]@', '', $board_value);
    // Multiple empty squares.
    $replace_pairs = array();
    for ($i=10; $i >= 0; $i--) {
      $replace_pairs[$i] = str_repeat('-', $i);
    }
    $board_value = strtr($board_value, $replace_pairs);
    $item['piece_placement'] = new PiecePlacement($board_value, $item['file_max']);

    return $item;
  }

}
